#define FCY 80000000/2
#include <p33FJ64GS606.h>

/* 
 * File:   SPIDevice.c
 * Created on October 29, 2018, 2:49 PM
 */
#include <libpic30.h>
#include <xc.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#pragma config ICS = PGD1 
#pragma config JTAGEN = OFF // JTAG Enable bit (JTAG is disabled)
// FPOR
#pragma config FPWRT = PWR128 // sets POR reset time to 128 us
// FWDT
#pragma config WDTPOST = PS32768 // Watchdog Timer Postscaler bits (1:32,768)
#pragma config WDTPRE = PR128 // Watchdog Timer Prescaler bit (1:128)

#pragma config WINDIS = OFF 
#pragma config FWDTEN = OFF 
// FOSC
#pragma config POSCMD = NONE 
#pragma config OSCIOFNC = ON // OSC2 Pin is digital output pin
#pragma config FCKSM = CSECMD 
// FOSCSEL

// FRC clock is nominally 7.37 MHz
#pragma config FNOSC = FRC 
#pragma config IESO = ON
#pragma config GWRP = OFF 

#define BAUDRATE 57600
#define U2TX_PIN TRISFbits.TRISF5   // UART TX pin
#define U2RX_PIN TRISFbits.TRISF4   // UART RX pin
#define M_PI 3.145926538979323846
#define U2TX LATFbits.LATF5 // transmit
#define U2RX PORTFbits.RF4  // receive
// PLL CONFIGURATUION
void init_PWM(void);
void openPinky(void); //4
void closePinky(void); //4
void openRing(void); //3
void closeRing(void); //3
void openMiddle(void); //2
void closeMiddle(void); //2
void openIndex(void); //1
void closeIndex(void); //1
void open(void); // function for getting into the open hand gesture
void close(void); 
int state; // 1 is fine pinch, 2 is point, 3 is close, & 4 is open
int init = 1;
void initPLL(void){
// http://microchipdeveloper.com/16bit:osc
// N1 = PLLPRE + 2
// N2 = 2 * (PLLPOST + 1)
// N3 = ?
// M = PLLDIV + 2
//
// 8 MHz < FPLLI < 64 MHz : = 7.37 MHz ~ 8 MHz
// 8 MHz < FPFD < (FVCO/16)) MHz : FPFD = FPLLI / N1 ?
// 400 MHz < FVCO < 1600 MHz : FVCO = FPLLI(M/N1)
// FPLLO = FPLLI (M / (N1 * N2))
// FPLLI = 8 MHz,
// FPLLO = 80 MHz = 8 MHz (M / N1 * N2)
// So 7.37 MHz * (M/N1 * N2) = 160 MHz
// N1 = N2 = 2, M/4 = 21.70 --> M = 87
PLLFBDbits.PLLDIV = 41; // M = 43
CLKDIVbits.PLLPRE = 0; // N1 = 2
CLKDIVbits.PLLPOST = 0; // N2 = 2
// Initiate Clock Switch to primary oscillator with PLL (NOSC=0b001)
__builtin_write_OSCCONH(0x01);
__builtin_write_OSCCONL(0x01); // Enable Switch and sec osc RTCC
// Wait for Clock switch to occur
while (OSCCONbits.COSC != 0x01);
// Wait for PLL to lock
while (OSCCONbits.LOCK != 1);
}
void initUART2(void) { 
    int BRG = (FCY/BAUDRATE/4) - 1;
    U2MODEbits.BRGH = 1;    // enable High Speed Mode
    U2BRG = BRG;            // store the value in the register
    
    U2MODEbits.PDSEL = 0;   // 8 bit data, no parity
    U2MODEbits.STSEL = 0;   // 1 stop bit
    U2MODEbits.UEN = 0b00;  // UxTX and UxRX pings are enabled
    U2MODEbits.UARTEN = 1;  // enable UART RX/TX
    U2STAbits.UTXEN = 1;    // enable the transmitter
    
    U2TX_PIN = 0;
    U2RX_PIN = 1;        
}

void outUART(char c) {
    // wait for transmit buffer to be empty
    while (U2STAbits.UTXBF);
    U2TXREG = c;
}

uint8_t inUART(void) {
    // while data is not received yet
    while (!U2STAbits.URXDA);
    // return data in receive buffer
    return (U2RXREG);
}
int SerialTransmit(unsigned char *buffer)
{
    unsigned int size = strlen(buffer);
    while( size)
    {
        while( U2STAbits.UTXBF);    // wait while TX buffer full
        U2TXREG = *buffer;          // send single character to transmit buffer
 
        buffer++;                   // transmit next character on following loop
        size--;                     // loop until all characters sent (when size = 0)
    }
 
    while( !U2STAbits.TRMT);        // wait for last transmission to finish
 
    return 0;
}
int power(int b, unsigned int exp) {
    int i, result = 1;
    for (i = 0; i < exp; i++)
        result *= b;
    return result;
 }
void convertToBinary(int *output, unsigned char byte) {
    //int output[__CHAR_BIT__];
    int i;
    int r;
   
   int counter=0;
   for(i = 7; i >= 0; i-- ) {
     r= (( byte >> i ) & 1 ? 1 : 0 );
     //char str=r+'0';
     //outUART(str);
     output[counter]=r;
 
    counter++;
}
    
}
//converts it to one 24 bit number
void bitShifter(int *output, int *byte1,int *byte2,int *byte3) {
    
    int i;
    for(i = 0; i < 24; i++){
        if(i <= 7){
            output[i] = byte1[i];
        }
        else if (i < 16){
            output[i] = byte2[i-8];
        }
        else{
            output[i] = byte3[i-16];
        }
    }
    
}
float formula(float mean, float sd, float num){
    
//    char bufe[23] = {'0'};
//    sprintf(bufe,"%f", (((1)/(sqrtf(2*M_PI*sd*sd))) * (expf(-((num-mean)*(num-mean))/(2*(sd*sd)))))*100000);
//    SerialTransmit(bufe);
//    SerialTransmit(",");
    
    //THERE ARE SOME 0s
    return (((1)/(sqrtf(2*M_PI*sd*sd))) * (expf(-((num-mean)*(num-mean))/(2*(sd*sd)))))*100000+0.000001;
    //return ((1)/(sqrtf(2*3.14159*sd*sd)))* (expf(-(num-mean)*(num-mean)/(2*(sd*sd))));
    
}
float getVal(float *mean,float *sd, float *new_value){
    
    float val = 1.0;
    int i;
    for (i = 0; i < 8; i++){
        if(sd[i] == 0){
            val = val * 1; 
        }
        else{
            val = val * (formula(mean[i], sd[i], new_value[i]));
        }
    }
//    char bufe[23] = {'0'};
//    SerialTransmit("|");
//    sprintf(bufe,"%f", val);
//    SerialTransmit(bufe);
//    SerialTransmit("|");
    return val;
}
int prediction(int *output) {
    
    
    float std [4][8] = {{25.68258495313132, 28.50856243958358, 1.4210854715202004e-14, 1.4210854715202004e-14, 24.47847090530435, 1.4210854715202004e-14, 18.22420561935011, 25.025243038067114},
        {25.68258495313132, 28.50856243958358, 1.4210854715202004e-14, 1.4210854715202004e-14, 24.47847090530435, 1.4210854715202004e-14, 18.22420561935011, 25.025243038067114},
        {0.8460200683664899, 0.7967925668794608, 0.9540620561335411, 0.6510922691714593, 0.6487870428682744, 1.4210854715202004e-14, 0.7103042529658954, 0.5189165181048296},
        {0.8460200683664899, 0.7967925668794608, 0.9540620561335411, 0.6510922691714593, 0.6487870428682744, 1.4210854715202004e-14, 0.7103042529658954, 0.5189165181048296}};
    
    float mean [4][8] = {{113.78510377551021, 113.57865734693877, 83.88608999999998, 83.88608999999998, 111.98827367346938, 83.88608999999998, 102.43684102040814, 112.95455857142856},
        {113.78510377551021, 113.57865734693877, 83.88608999999998, 83.88608999999998, 111.98827367346938, 83.88608999999998, 102.43684102040814, 112.95455857142856},
        {26.5502354, 23.0432542, 35.5263468, 34.053925400000004, 4.5012768, 83.88608999999998, 166.0251816, 4.2961286},
        {26.5502354, 23.0432542, 35.5263468, 34.053925400000004, 4.5012768, 83.88608999999998, 166.0251816, 4.2961286}};
    
    
    float vals [4] = {   
        getVal(mean[0], std[0],output),
        getVal(mean[1], std[1],output),
        getVal(mean[2], std[2],output),
        getVal(mean[3], std[3],output)
    };
    
    int i;
    float maxVal = -10000000.0;
    int maxIndex = 0;
    
    float minVal = 10000000000.0;
    int minIndex = 0;
    for (i = 0; i < 4; i++){
        if (vals[i] < minVal){
            minVal = vals[i];
            minIndex = i;
        }
        if (vals[i]>maxVal){
            maxVal = vals[i];
            maxIndex = i;
        }
    }
        
    if(vals[maxIndex]/(vals[0]+vals[1]+vals[2]+vals[3]) > 0){
        return minIndex+1;
    }
                
    return 0;
}
float sample_Point[8];
int variables[100];
int ctr;
int crawler;
int overallCounter;
void movement(int gesture){
    
    
}
int mode(int a[],int n) {
   int maxValue = 0, maxCount = 0, i, j;

   for (i = 0; i < n; ++i) {
      int count = 0;
      
      for (j = 0; j < n; ++j) {
         if (a[j] == a[i])
         ++count;
      }
      
      if (count > maxCount) {
         maxCount = count;
         maxValue = a[i];
      }
   }

   return maxValue;
}
void uart_Send(unsigned char *byte){ //unsigned char byte1,unsigned char byte2, unsigned char byte3

    int i;
    int training = 1; // Training = 0
    //SerialTransmit("Start");
    
        int byte1_bin[__CHAR_BIT__-1] = {0};
        int byte2_bin[__CHAR_BIT__-1] = {0};
        int byte3_bin[__CHAR_BIT__-1] = {0};
        int byte4_bin[__CHAR_BIT__-1] = {0};
        int byte5_bin[__CHAR_BIT__-1] = {0};
        int byte6_bin[__CHAR_BIT__-1] = {0};
        int byte7_bin[__CHAR_BIT__-1] = {0};
        int byte8_bin[__CHAR_BIT__-1] = {0};
        int byte9_bin[__CHAR_BIT__-1] = {0};
        int byte10_bin[__CHAR_BIT__-1] = {0};
        int byte11_bin[__CHAR_BIT__-1] = {0};
        int byte12_bin[__CHAR_BIT__-1] = {0};
        int byte13_bin[__CHAR_BIT__-1] = {0};
        int byte14_bin[__CHAR_BIT__-1] = {0};
        int byte15_bin[__CHAR_BIT__-1] = {0};
        int byte16_bin[__CHAR_BIT__-1] = {0};
        int byte17_bin[__CHAR_BIT__-1] = {0};
        int byte18_bin[__CHAR_BIT__-1] = {0};
        int byte19_bin[__CHAR_BIT__-1] = {0};
        int byte20_bin[__CHAR_BIT__-1] = {0};
        int byte21_bin[__CHAR_BIT__-1] = {0};
        int byte22_bin[__CHAR_BIT__-1] = {0};
        int byte23_bin[__CHAR_BIT__-1] = {0};
        int byte24_bin[__CHAR_BIT__-1] = {0};
        convertToBinary(byte1_bin, byte[0]);
        //outUART('\n');
        convertToBinary(byte2_bin, byte[1]);
        //outUART('\n');
        convertToBinary(byte3_bin, byte[2]);
        //outUART('\n');
        convertToBinary(byte4_bin, byte[3]);
        //outUART('\n');
        convertToBinary(byte5_bin, byte[4]);
        //outUART('\n');
        convertToBinary(byte6_bin, byte[5]);
        //outUART('\n');
        convertToBinary(byte7_bin, byte[6]);
        //outUART('\n');
        convertToBinary(byte8_bin, byte[7]);
        //outUART('\n');
        convertToBinary(byte9_bin, byte[8]);
        //outUART('\n');
        convertToBinary(byte10_bin, byte[9]);
        //outUART('\n');
        convertToBinary(byte11_bin, byte[10]);
        //outUART('\n');
        convertToBinary(byte12_bin, byte[11]);
        //outUART('\n');
        convertToBinary(byte13_bin, byte[12]);
        //outUART('\n');
        convertToBinary(byte14_bin, byte[13]);
        //outUART('\n');
        convertToBinary(byte15_bin, byte[14]);
        //outUART('\n');
        convertToBinary(byte16_bin, byte[15]);
        //outUART('\n');
        convertToBinary(byte17_bin, byte[16]);
        //outUART('\n');
        convertToBinary(byte18_bin, byte[17]);
        //outUART('\n');
        convertToBinary(byte19_bin, byte[18]);
        //outUART('\n');
        convertToBinary(byte20_bin, byte[19]);
        //outUART('\n');
        convertToBinary(byte21_bin, byte[20]);
        //outUART('\n');
        convertToBinary(byte22_bin, byte[21]);
        //outUART('\n');
        convertToBinary(byte23_bin, byte[22]);
        //outUART('\n');
        convertToBinary(byte24_bin, byte[23]);
        //outUART('\n');
        //shift the bits
        int shifted1[23] = {0};
        int shifted2[23] = {0};
        int shifted3[23] = {0};
        int shifted4[23] = {0};
        int shifted5[23] = {0};
        int shifted6[23] = {0};
        int shifted7[23] = {0};
        int shifted8[23] = {0};
        //char buf[7] = {'0'};
        //sprintf(buf,"%d", byte1_bin[0]);
        
        
        //SerialTransmit(buf); //sould be 92
        
        bitShifter(shifted1,byte1_bin, byte2_bin, byte3_bin);
        bitShifter(shifted2,byte4_bin, byte5_bin, byte6_bin);
        bitShifter(shifted3,byte7_bin, byte8_bin, byte9_bin);
        bitShifter(shifted4,byte10_bin, byte11_bin, byte12_bin);
        bitShifter(shifted5,byte13_bin, byte14_bin, byte15_bin);
        bitShifter(shifted6,byte16_bin, byte17_bin, byte18_bin);
        bitShifter(shifted7,byte19_bin, byte20_bin, byte21_bin);
        bitShifter(shifted8,byte22_bin, byte23_bin, byte24_bin);
        //char buf[23] = {'0'};

        
        //converts it back to a decimal
        int r;
        float num1 = 0;
        for (r = 0; r <= 23; r++){
             num1 = num1 + (shifted1[r]*pow(2,(23-r)));
             
        }
        float num2 = 0;
        for (r = 0; r <= 23; r++){
             num2 = num2 + (shifted2[r]*pow(2,(23-r)));
             
        }
        float num3 = 0;
        for (r = 0; r <= 23; r++){
             num3 = num3 + (shifted3[r]*pow(2,(23-r)));
             
        }
        float num4 = 0;
        for (r = 0; r <= 23; r++){
             num4 = num4 + (shifted4[r]*pow(2,(23-r)));
             
        }
        float num5 = 0;
        for (r = 0; r <= 23; r++){
             num5 = num5 + (shifted5[r]*pow(2,(23-r)));
             
        }
        float num6 = 0;
        for (r = 0; r <= 23; r++){
             num6 = num6 + (shifted6[r]*pow(2,(23-r)));
             
        }
        float num7 = 0;
        for (r = 0; r <= 23; r++){
             num7 = num7 + (shifted7[r]*pow(2,(23-r)));
             
        }
        float num8 = 0;
        for (r = 0; r <= 23; r++){
             num8 = num8 + (shifted8[r]*pow(2,(23-r)));
             
        }
        char buf1[23] = {'0'};
        char buf2[23] = {'0'};
        char buf3[23] = {'0'};
        char buf4[23] = {'0'};
        char buf5[23] = {'0'};
        char buf6[23] = {'0'};
        char buf7[23] = {'0'};
        char buf8[23] = {'0'};
        
//
        sprintf(buf1,"%f", num1);
        sprintf(buf2,"%f", num2);
        sprintf(buf3,"%f", num3);
        sprintf(buf4,"%f", num4);
        sprintf(buf5,"%f", num5);
        sprintf(buf6,"%f", num6);
        sprintf(buf7,"%f", num7);
        sprintf(buf8,"%f", num8);
        
        //an array of 8 buffs is a sample data point
        //int training = 0; // 0 is training 
        if (training == 1){
            //figure out how to create the sample_Point array
            sample_Point[0]=num1/100000;
            sample_Point[1]=num2/100000;
            sample_Point[2]=num3/100000;
            sample_Point[3]=num4/100000;
            sample_Point[4]=num5/100000;
            sample_Point[5]=num6/100000;
            sample_Point[6]=num7/100000;
            sample_Point[7]=num8/100000;
            ctr = ctr + 1;
        }else{
           
            SerialTransmit(buf1);
            SerialTransmit(",");
            SerialTransmit(buf2);
            SerialTransmit(",");
            SerialTransmit(buf3);
            SerialTransmit(",");
            SerialTransmit(buf4);
            SerialTransmit(",");
            SerialTransmit(buf5);
            SerialTransmit(",");
            SerialTransmit(buf6);
            SerialTransmit(",");
            SerialTransmit(buf7);
            SerialTransmit(",");
            SerialTransmit(buf8);
            SerialTransmit(",");
            SerialTransmit("\n");
//            crawler = crawler + 1;
//            if (i%8 == 0){
//                SerialTransmit("\n");
//                crawler = 0;
//            }
        }
    
    //SerialTransmit("\n");
    //predict
    //ctr = 0;
    if(training == 1){
        variables[overallCounter] = prediction(sample_Point);
        overallCounter = overallCounter + 1;

        if(overallCounter == 100){
            overallCounter = 0;
            int tValue = mode(variables, 100);
            //new
            int i;
            for(i =0; i < 100; i++){
                if (variables[i] != 0){
                    tValue = variables[i];     
                }
            }
            //end new
            
            char bufe[23] = {'0'};
            sprintf(bufe,"%d", tValue);
            SerialTransmit(bufe);
            for (i =0; i < 10; i++){
                variables[i] = 0;
            }
            
        if(tValue == 0){
            if (state == 1) {
                open();
            }
            state = 2;
        }

        if(tValue == 1){
            if (state == 2) {
                close();
            }
            state = 1;
 
        }
        if(tValue == 2){
           if (state == 2) {
                close();
            }
            state = 1;
        }
        if(tValue == 3){
            if (state == 2) {
                close();
            }
            state = 1;
        }
        //}

        //grab more data
        //sample_Point = {0};
        ctr = 0;
        }
    }
} 
void init_SPI(void) {
    
    SPI1CON1bits.DISSCK = 0;                // Internal serial clock is enabled
    SPI1CON1bits.DISSDO = 0;                // SDO1 pin is controlled by the SPI module
    SPI1CON1bits.MODE16 = 0;                // 8-bit communication
    SPI1CON1bits.MSTEN = 1;                 // Master mode enabled
    SPI1CON1bits.SMP = 0;                   // Input data is sampled at the middle of data output time
    SPI1CON1bits.SSEN = 0;                  // SS1 is controlled by the PORT function
    SPI1CON1bits.CKE = 0;                    // Serial output data changes on transition
    SPI1CON1bits.CKP = 0;                   // Idle state for clock is low level; active high
    SPI1CON1bits.PPRE = 0b10;               // Primary prescaler 16:1
    SPI1CON1bits.SPRE = 0b101;              // Secondary prescaler 4:1
    SPI1STATbits.SPIEN = 1;                 // Enable SPI module
    
}

void configpins(void)
{
    TRISBbits.TRISB14=0; //Makes RB8 an output (Chip select)
    TRISDbits.TRISD8=0; //Reset
    TRISDbits.TRISD9=0; //Start
    TRISDbits.TRISD10=0; //PWDN
    TRISDbits.TRISD11=1; //DRDY
    
    
    
    
}

unsigned char Transfer(unsigned char data)
{   
    SPI1STATbits.SPIRBF=0; //Clears the receives buffer bit 
    SPI1BUF = data;//Puts data into the buffer
    //SPI1STATbits.=0;//Turns off buffer enable 
    while (!SPI1STATbits.SPIRBF);//While the Receive buffer flag is empty
    return SPI1BUF; // read a byte
    
}
void openPinky() //4
{
    PHASE4 = 1920;
    SPHASE4 = 1920;
    
    PWMCON4bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON4bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON4bits.PENH = 1;      				/* PWM4H is controlled by PWM module */ 
    IOCON4bits.PENL = 1;      				/* PWM4L is controlled by PWM module */
    IOCON4bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC4 = 384;                            /* 50% Duty cycle for PWM1H */ 
	SDC4 = 1920;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void closePinky() //4
{
    PHASE4 = 1920;
    SPHASE4 = 1920;
    
    PWMCON4bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON4bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON4bits.PENH = 1;      				/* PWM4H is controlled by PWM module */ 
    IOCON4bits.PENL = 1;      				/* PWM4L is controlled by PWM module */
    IOCON4bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC4 = 1920;                            /* 50% Duty cycle for PWM1H */ 
	SDC4 = 384;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void openRing() //3
{
    PHASE3 = 1920;
    SPHASE3 = 1920;
    
    PWMCON3bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON3bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON3bits.PENH = 1;      				/* PWM1H is controlled by PWM module */ 
    IOCON3bits.PENL = 1;      				/* PWM1L is controlled by PWM module */
    IOCON3bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC3 = 384;					   			/* 50% Duty cycle for PWM1H */ 
	SDC3 = 1920;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void closeRing() //3
{
    PHASE3 = 1920;
    SPHASE3 = 1920;
    
    PWMCON3bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON3bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON3bits.PENH = 1;      				/* PWM1H is controlled by PWM module */ 
    IOCON3bits.PENL = 1;      				/* PWM1L is controlled by PWM module */
    IOCON3bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC3 = 1920;					   			/* 50% Duty cycle for PWM1H */ 
	SDC3 = 384;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void openMiddle() //2
{
    PHASE2 = 1920;
    SPHASE2 = 1920;
    
    PWMCON2bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON2bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON2bits.PENH = 1;      				/* PWM1H is controlled by PWM module */ 
    IOCON2bits.PENL = 1;      				/* PWM1L is controlled by PWM module */
    IOCON2bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC2 = 384;					   			/* 50% Duty cycle for PWM1H */ 
	SDC2 = 1920;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void closeMiddle() //2
{
    PHASE2 = 1920;
    SPHASE2 = 1920;
    
    PWMCON2bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON2bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON2bits.PENH = 1;      				/* PWM1H is controlled by PWM module */ 
    IOCON2bits.PENL = 1;      				/* PWM1L is controlled by PWM module */
    IOCON2bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC2 = 1920;					   			/* 50% Duty cycle for PWM1H */ 
	SDC2 = 384;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void openIndex() //1
{
    PHASE1 = 1920;                          /* PWM1H, PHASEx = ((1 / 500kHz) / 1.04ns) = 1923, where 500kHz 
											 is the desired switching frequency and 1.04ns is PWM resolution. */
    SPHASE1 = 1920;                         /* PWM1L, SPHASEx = ((1 / 250kHz) / 1.04ns) = 3846, where 250kHz 
											 is the desired switching frequency and 1.04ns is PWM resolution. */
    PWMCON1bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON1bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON1bits.PENH = 1;      				/* PWM1H is controlled by PWM module */ 
    IOCON1bits.PENL = 1;      				/* PWM1L is controlled by PWM module */
    IOCON1bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC1 = 384;					   			/* 50% Duty cycle for PWM1H */ 
	SDC1 = 1920;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void closeIndex() //1
{
    PHASE1 = 1920;                          /* PWM1H, PHASEx = ((1 / 500kHz) / 1.04ns) = 1923, where 500kHz 
											 is the desired switching frequency and 1.04ns is PWM resolution. */
    SPHASE1 = 1920;                         /* PWM1L, SPHASEx = ((1 / 250kHz) / 1.04ns) = 3846, where 250kHz 
											 is the desired switching frequency and 1.04ns is PWM resolution. */
    PWMCON1bits.ITB = 1;					/* Use Phasex and SPHASEx registers for Independent time base */
	PWMCON1bits.DTC = 2;					/* Disable Deadtime */
    
    IOCON1bits.PENH = 1;      				/* PWM1H is controlled by PWM module */ 
    IOCON1bits.PENL = 1;      				/* PWM1L is controlled by PWM module */
    IOCON1bits.PMOD = 3;      				/* Select True Independent Output PWM mode */
    
    PDC1 = 1920;					   			/* 50% Duty cycle for PWM1H */ 
	SDC1 = 384;							/* 50% Duty cycle for PWM1L */
    
    PTCONbits.PTEN       = 1;      			/* Enable the PWM Module */
}

void open() //opens all [1,2,3,4,5,6]
{
    openIndex();
    openMiddle();
    openPinky();
    __delay_ms(25);
    openRing();
    __delay_ms(325);
    PTCONbits.PTEN = 0;
}
void close() // closes all [1,2,3,4,5,6]
{
    
    closePinky();
    closeMiddle();
    closeIndex();
    __delay_ms(25);
    closeRing();
    __delay_ms(325);
    PTCONbits.PTEN = 0;
}
void init_PWM()
{
    /* Configure Oscillator to operate the device at 40Mhz
	   Fosc= Fin*M/(N1*N2), Fcy=Fosc/2
 	   Fosc= 7.37*(43)/(2*2)=80Mhz for Fosc, Fcy = 40Mhz */

	/* Configure PLL prescaler, PLL postscaler, PLL divisor */
	 //PLLFBD=41; 				/* M = PLLFBD + 2 */
	CLKDIVbits.PLLPOST=0;   /* N1 = 2 */
	CLKDIVbits.PLLPRE=0;    /* N2 = 2 */

    __builtin_write_OSCCONH(0x01);			/* New Oscillator FRC w/ PLL */
    __builtin_write_OSCCONL(0x01);  		/* Enable Switch */
      
	while(OSCCONbits.COSC != 0b001);		/* Wait for new Oscillator to become FRC w/ PLL */  
    while(OSCCONbits.LOCK != 1);			/* Wait for Pll to Lock */

	/* Now setup the ADC and PWM clock for 120MHz
	   ((FRC * 16) / APSTSCLR ) = (7.37 * 16) / 1 = ~ 120MHz*/

	ACLKCONbits.FRCSEL = 1;					/* FRC provides input for Auxiliary PLL (x16) */
	ACLKCONbits.SELACLK = 1;				/* Auxiliary Oscillator provides clock source for PWM & ADC */
	ACLKCONbits.APSTSCLR = 7;				/* Divide Auxiliary clock by 1 */
	ACLKCONbits.ENAPLL = 1;					/* Enable Auxiliary PLL */
	
	while(ACLKCONbits.APLLCK != 1);			/* Wait for Auxiliary PLL to Lock */
    state = 1; // starts from the "closed" position
}

void ReadDevice(void)//Gets the Device ID
{
    PORTBbits.RB14=0;//Clears the chip select
    Transfer(0x11);
    Transfer(0x20);
    Transfer(0x00);
    Transfer(0x00);
    //Transfer(0x00);
   // Transfer(0x00);
   // Transfer(0x00);
   // Transfer(0x00);

    PORTBbits.RB14=1;//Sets the Device ID
}
int main(int argc, char** argv) 
{
    initPLL();
    initUART2();
    configpins();
    init_SPI();
    if(init)
    {
    init_PWM();
    init = 0;
    }
    crawler = 0;
    overallCounter = 0;
    int i;
    for (i =0; i < 100; i++){
        variables[i] = 0;
    }
    ctr = 0;
    sample_Point[0]=0;
    sample_Point[1]=0;
    sample_Point[2]=0;
    sample_Point[3]=0;
    sample_Point[4]=0;
    sample_Point[5]=0;
    sample_Point[6]=0;
    sample_Point[7]=0;
    char   buf[1024]; 
    PORTDbits.RD8=1; //reset//
    PORTDbits.RD9=0; //Start
    PORTDbits.RD10=1; //PWDN
    //Transfer(0x11);
    int count=0;
    while (count<100){
        count=count+1;
    }
    PORTDbits.RD8=1; //reset
    PORTDbits.RD9=1; //Start
    PORTDbits.RD10=1;
    count=0;
    while (count<100){
        count=count+1;
    }
    PORTDbits.RD9=0;
    PORTDbits.RD8=0;
    count=0;
    while (count<100){
        count=count+1;
    }
    PORTDbits.RD8=1;
    count=0;
     while (count<100){
        count=count+1;
    }
    PORTDbits.RD9=1;
    PORTBbits.RB14=1;
    int InitialV=0;
    while(1)// Forever loop
    {
     //unsigned char Result=SPITransaction('W');//Sends W to the buffer and gets back a byte
        if (PORTDbits.RD11==0) {
        count=0;
        PORTBbits.RB14=0;
        //getting the stuff we dont need, out
        Transfer(0x12);
        Transfer(0x00);
        Transfer(0x00);
        Transfer(0x00);
        //getting the stuff we need
        unsigned char val[23];
        int i;
        for(i = 0; i < 24; i++){
            val[i] =  Transfer(0x00);
        }
        uart_Send(val);
     
        
        count=count+1;
    //}
        if (InitialV==0){
            PORTBbits.RB14=0;
            Transfer(0x11);
            Transfer(0x41);
            Transfer(0x00);
            Transfer(0x06);
            Transfer(0x11);
            Transfer(0x45);
            Transfer(0x00);
            Transfer(0x00);
            Transfer(0x11);
            Transfer(0x43);
            Transfer(0x00);
            Transfer(0xC0);
            
            PORTBbits.RB14=1;
            InitialV=1;
        }

        PORTBbits.RB14=1;
        }
      //ReadDevice();//Gets the Device ID
    //PORTBbits.RB8=0;//Clears the chip select
    //Transfer(0x08);//Bytes to get the device id
    //PORTBbits.RB8=1;//Sets the Device ID
    }
    
}